@@ -1,7 +1,7 @@
 # Starfish - Perl-based System for Text-Embedded
 #     Programming and Preprocessing
 #
-# (c) 2001-2016 Vlado Keselj http://web.cs.dal.ca/~vlado vlado@dnlp.ca
+# (c) 2001-2017 Vlado Keselj http://web.cs.dal.ca/~vlado vlado@dnlp.ca
 #               and contributing authors
 #
 # See the documentation following the code.  You can also use the
@@ -21,7 +21,8 @@ use vars qw($VERSION @ISA @EXPORT @EXPORT_OK %EXPORT_TAGS); # Exporter vars
   appendfile echo file_modification_date 
   file_modification_time getfile getmakefilelist get_verbatim_file
   getinclude include
-  last_update putfile read_records read_starfish_conf starfish_cmd ) ] );
+  last_update putfile read_records read_starfish_conf
+  sfish_add_tag starfish_cmd ) ] );
 @EXPORT_OK = ( @{ $EXPORT_TAGS{'all'} } );
 @EXPORT = @{ $EXPORT_TAGS{'all'} };
 
@@ -29,7 +30,7 @@ use vars qw($VERSION @ISA @EXPORT @EXPORT_OK %EXPORT_TAGS); # Exporter vars
 use vars qw($NAME $ABSTRACT $VERSION);
 $NAME     = 'Starfish';
 $ABSTRACT = 'Perl-based System for Text-Embedded Programming and Preprocessing';
-$VERSION  = '1.22';
+$VERSION  = '1.23';
 
 use vars qw($Revision);
 ($Revision = substr(q$Revision: 392 $, 10)) =~ s/\s+$//;
@@ -273,15 +274,16 @@ sub digest {
     }
     
     if (defined $self->{macrosdefined}) {
-	my ($m, $s);
-	while (($m,$s) = each %{$self->{Macros}}) {
-	    if ($s =~ /\n/) {
-		my $p1 = "$`$&"; $s = $';
-		if ($s) { $s = $p1.wrap($s) }
-		else { $s = $p1 }
-	    }
-	    $self->{Out}.= $self->{MprefAuxDefine}.$s.$self->{MsufAuxDefine};
+      my ($m, $s);
+      for $m (sort keys %{$self->{Macros}}) {
+	$s = $self->{Macros}->{$m};
+	if ($s =~ /\n/) {
+	  my $p1 = "$`$&"; $s = $';
+	  if ($s) { $s = $p1.wrap($s) }
+	  else { $s = $p1 }
 	}
+	$self->{Out}.= $self->{MprefAuxDefine}.$s.$self->{MsufAuxDefine};
+      }
     }
 }
 
@@ -829,6 +831,27 @@ sub setStyle {
     $self->{Style} = $s;
 }
 
+sub sfish_add_tag($$) { $::Star->add_tag(@_) }
+
+# adds tags such as %slide:.* and %<slide> by adding appropriate hooks
+# eg: add_tag('slide', 'ignore')
+# eg: add_tag('sl,l',  'echo')
+sub add_tag {
+  my $self = shift;
+  if (ref($self) ne 'Text::Starfish')
+  { unshift @_, $self; $self = $::Star; }
+  my $tag = shift; my $fun = shift;
+  my $lc = $self->{LineComment};
+  #die "tag=($tag) fun=($fun) ref(fun)=".ref($fun);
+  if (ref($fun) eq '') {
+    if    ( $fun eq 'ignore') { $fun = sub{''} }
+    elsif ( $fun eq 'echo') { $fun = sub{$_[2]} }
+  }
+  my $lc1 = ($lc eq '') ? $lc : "$lc?";
+  $self->addHook(qr/$lc1<$tag>\n?((?:.|\n)*?)$lc1<\/$tag>\n?/, $fun);
+  $self->addHook(qr/$lc$tag:(.*\n)/, $fun);
+}
+
 sub add_hook {
     my $self = shift; my $ht = shift;
     my $hooks = $self->{hook}; my $hook = { ht=>$ht };
@@ -1505,6 +1528,15 @@ C<-copyhooks> Copies hooks from the Star object (C<$::Star>).  This
     C<-copyhooks> copies the fields: C<Style>, C<CodePreparation>,
     C<LineComment>, and per-component copies the array C<hook>.
 
+=head2 $o->add_tag($tag, $action)
+
+Normally used by C<sfish_add_tag> by translating the call to
+C<$Star->add_tag($tag, $action)>.  Examples:
+
+  $Star->add_tag('slide', 'ignore');
+  $Star->add_tag('slide', 'echo');
+
+See C<sfish_add_tag> for a few more details.
 
 =head2 $o->add_hook($ht,...)
 
@@ -1696,6 +1728,29 @@ executing the configuration files in the order from first ancestor
 down.  For each file, it changes directory to the corresponding
 directory, and requires (in Perl style) the file in the package main.
 
+=head2 sfish_add_tag ( I<tag>, I<action> )
+
+Used to introduce simple tags such as line tag C<%sl,l:> and
+%<sl,l>...</sl,l> in TeX/LaTeX for inclusion and exclusion of text.
+Example:
+
+     sfish_add_tag('sl,l', 'echo');
+     sfish_add_tag('slide', 'ignore');
+
+and, for example, the following text is included:
+
+     %sl,l:some text to the end of line
+     %<sl,l>
+     more lines of text
+     %</sl,l>
+
+and the following text is excluded:
+
+     %slide:this line is excluded
+     %<slide>
+     more lines of text excluded
+     %</slide>
+
 =head2 starfish_cmd I<list of file names and options>
 
 The function C<starfish_cmd> is called by the script C<starfish> with
@@ -1963,4 +2018,4 @@ interface.
 =back
 
 =cut
-# $Id: Starfish.pm 251 2016-05-17 01:34:16Z vlado $
+# $Id: Starfish.pm 252 2016-05-17 02:04:10Z vlado $
